<?php

namespace HCM\AVL\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;

class AnaliseApuracaoController extends Crud {
    protected $dataSourceName = 'getAval';
    protected $dataSourceNameAvalPendentes = 'getAvalPendentes';
    protected $dataSourceNameColabPendentes = 'getColaboradoresPendentes';
    protected $dataSourceNamePendencias = 'getPendencias';
    protected $dataSourceNameColabAptos = 'getColaboradoresAptos';
    protected $dataSourceNameParticipApurados = 'getParticipantesApurados';
    protected $dataSourceNameApuracaoAgrupamento = 'getApuracaoAgrupamento';
    protected $dataSourceNameApuracaoQuestoes = 'getApuracaoQuestoes';
    protected $dataSourceNameApuracaoResultadoGeral = 'getResultadoGeral';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    protected $AnaliseApuracaoService;
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\AVL\Services\AnaliseApuracaoService $AnaliseApuracaoService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->AnaliseApuracaoService = $AnaliseApuracaoService;
        parent::__construct($dataSourceManager);
    }
    
    public function getAval(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getAvalPendentes(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRAVALIACAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameAvalPendentes);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getColaboradoresPendentes(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRAVALIACAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameColabPendentes);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getColaboradoresAptos(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRAVALIACAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameColabAptos);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getPendencias(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRAVALIACAO', 'P_NRPARCEXECAVAL', 'P_NRCOMBQUESTAVAL');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNamePendencias);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getParticipantesApurados(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRAVALIACAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameParticipApurados);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getApuracaoAgrupamento(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRAVALIACAO', 'P_NRPARCEXECAVAL', 'P_NRPARCAVALIADO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameApuracaoAgrupamento);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function getApuracaoQuestoes(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRAVALIACAO', 'P_NRPARCEXECAVAL', 'P_NRPARCAVALIADO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameApuracaoQuestoes);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    
    public function getApuracaoPonderada(DTO\Request\Filter $request, DTO\Response $response) {
        
        try {

            $filters = $request->getFilterCriteria()->getConditions();
            $parameters = array();
            foreach ($filters as $filter) {
                unset($filter['operator']);
                foreach ($filter as $key => $value) {
                    if ($key == 'columnName') {
                        $parameters[$value] = $filter['value'];
                        unset($filter['columnName']);
                    }
                }
            }

            $resultado = $this->AnaliseApuracaoService->getApuracaoPonderada($parameters);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getApuracaoPonderada', $resultado));

        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getResultadoGeral(DTO\Request\Filter $request, DTO\Response $response) {
        
        $this->parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_NRAVALIACAO');
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameApuracaoResultadoGeral);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    
     protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
}