<?php
namespace HCM\BTL\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

use HCM\Geral\Services\UploadFileService;

class UploadsRepository {

    protected  $connection;
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection    = $connection;
        $this->entityManager = $entityManager;
    }

    /**
     * @return array
     */
    public function insertGpeAnexos($nranexo, $nrorg, $nrparcnegocio, $nmanexo, $dsreferencia, $type, $cdoperador, $path, $idfoto, $idDocumentoModelo = null, $nrtipodocumento = null, $nrcategdocumento = null, $iddocaprovado = UploadFileService::IDDOCAPROVADO_ONCREATE) {
        $dtatual = date("d/m/Y");
        $query = $this->entityManager->getConnection()->prepare("
            INSERT INTO GPE_ANEXOS 
                    (NRANEXO,
                    NRORG,
                    NRPARCNEGOCIO,
                    NMANEXO,
                    DSREFERENCIA,
                    IDTYPE,
                    DSPATH,
                    DTINCLUSAO,
                    DTULTATU,
                    CDOPERINCLUSAO,
                    CDOPERULTATU,
                    IDFOTO,
                    NRTIPODOCUMENTO,
                    NRCATEGDOCUMENTO,
                    IDDOCAPROVADO,
                    IDDOCUMENTOMODELO)
            VALUES (:NRANEXO,
                    :NRORG, 
                    :NRPARCNEGOCIO,
                    TO_CHAR('$nmanexo'),
                    :DSREFERENCIA,
                    :IDTYPE,
                    :DSPATH,
                    :DTATUAL,
                    :DTATUAL,
                    :CDOPERADOR,
                    :CDOPERADOR,
                    :IDFOTO,
                    :NRTIPODOCUMENTO,
                    :NRCATEGDOCUMENTO,
                    :IDDOCAPROVADO,
                    :IDDOCUMENTOMODELO)
        ");
       
        //$this->connection->execute();

        $query->execute(
            array(
              "NRORG"               => $nrorg,
              "NRPARCNEGOCIO"       => $nrparcnegocio,
              //"NMANEXO"           => $nmanexo,
              "NRANEXO"             => $nranexo,
              "DSREFERENCIA"        => $dsreferencia,
              "IDTYPE"              => $type,
              "DTATUAL"             => $dtatual,
              "CDOPERADOR"          => $cdoperador,
              "DSPATH"              => $path,
              "IDFOTO"              => $idfoto,
              "NRTIPODOCUMENTO"     => $nrtipodocumento,
              "NRCATEGDOCUMENTO"    => $nrcategdocumento,
              "IDDOCAPROVADO"       => $iddocaprovado,
              "IDDOCUMENTOMODELO"   => $idDocumentoModelo
            )
        );
    }
    
    public function getGpeAnexo($nranexo, $nrorg) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT NRANEXO, NRORG, NRPARCNEGOCIO, NMANEXO, DSREFERENCIA, DSPATH, IDFOTO FROM GPE_ANEXOS WHERE NRORG = :P_NRORG AND NRANEXO = :P_NRANEXO
        ");

        $query->execute(array(          
              "P_NRANEXO" => $nranexo,
              "P_NRORG"   => $nrorg,
            )
        );

        return $query->fetchAll();
    }
    
    public function getGpeAnexoParceiro($nrparceiro, $nrorg) {
        $query = $this->entityManager->getConnection()->prepare("
            SELECT * FROM GPE_ANEXOS WHERE NRORG = :P_NRORG AND NRPARCNEGOCIO = :P_NRPARCEIRO
        ");

        $query->execute(array(          
              "P_NRPARCEIRO" => $nrparceiro,
              "P_NRORG"      => $nrorg,
            )
        );

        return $query->fetchAll();
    }
    
    public function deleteAnexoExistenteParceiro($nranexo, $nrparceiro, $nrorg) {
        $query = $this->entityManager->getConnection()->prepare("
            DELETE FROM GPE_ANEXOS WHERE NRORG = :P_NRORG AND NRPARCNEGOCIO = :P_NRPARCEIRO AND NRANEXO = :P_NRANEXO
        ");

        return $query->execute(array(          
              "P_NRPARCEIRO" => $nrparceiro,
              "P_NRANEXO"    => $nranexo,
              "P_NRORG"      => $nrorg
            )
        );
    }
    
    public function updateGpeAnexos($nranexo, $nrorg, $nrparcnegocio, $nmanexo, $dsreferencia, $type, $cdoperador, $path, $idfoto) {
        $dtatual = date("d/m/Y");
        $query = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_ANEXOS
               SET NMANEXO = TO_CHAR('$nmanexo'),
                   DSREFERENCIA = :DSREFERENCIA, 
                   IDTYPE = :IDTYPE, 
                   DSPATH = :DSPATH, 
                   DSURL = :DSURL, 
                   DTULTATU = :DTATUAL, 
                   CDOPERULTATU = :CDOPERADOR
            WHERE NRANEXO = :NRANEXO 
              AND NRPARCNEGOCIO = :NRPARCNEGOCIO 
              AND IDFOTO = :IDFOTO
              AND NRORG = :NRORG
        ");
       
        //$this->connection->execute();

        $query->execute(
            array(
              "NRORG"         => $nrorg,
              "NRPARCNEGOCIO" => $nrparcnegocio,
              "NRANEXO"       => $nranexo,
              "DSREFERENCIA"  => $dsreferencia,
              "IDTYPE"        => $type,
              "DSURL"         => null,
              "DTATUAL"       => $dtatual,
              "CDOPERADOR"    => $cdoperador,
              "DSPATH"        => $path,
              "IDFOTO"        => $idfoto
            )
        );
    }
    
    /**
     *  Rotina responsável por aprovar/recusar documentos. Campo de aprovação: IDDOCAPROVADO
     * 
     *  @param  $nrorg
     *  @param  $nranexo
     *  @param  $status
     * 
     *  @return $this->entityManager->getConnection()->prepare()->execute()
     */
    public function updateIddocaprovadoGpeAnexo($nrorg, $nranexo, $status) {
        $query = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_ANEXOS
               SET  IDDOCAPROVADO = :P_IDDOCAPROVADO
               
            WHERE   NRANEXO     = :P_NRANEXO
                    AND NRORG   = :P_NRORG
        ");

        return $query->execute(
            array(
              "P_NRORG"             => $nrorg,
              "P_NRANEXO"           => $nranexo,
              "P_IDDOCAPROVADO"     => $status,
            )
        );
    }
    
    public function updateNomeGpeAnexos($nranexo, $nrorg, $nrparcnegocio, $dsreferencia) {
        $dtatual = date("d/m/Y");
        $query = $this->entityManager->getConnection()->prepare("
            UPDATE GPE_ANEXOS
               SET DSREFERENCIA = :DSREFERENCIA
            WHERE NRANEXO = :NRANEXO 
              AND NRPARCNEGOCIO = :NRPARCNEGOCIO 
              AND NRORG = :NRORG
        ");
        
        return $query->execute(
            array(
              "NRORG"         => $nrorg,
              "NRPARCNEGOCIO" => $nrparcnegocio,
              "NRANEXO"       => $nranexo,
              "DSREFERENCIA"  => $dsreferencia
            )
        );
    }
}