<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\TkEntity;
 
class FpaGrpitemEntity extends TkEntity {

    private $_nrGrpIncidAnterior = -1;
    
    const IDRETVALOR_NAO = 'N';
    const IDRETVALOR_SIM = 'S';
    const IDREFGERAR_PERCENTUAL = 'PERCENTUAL';
    const IDREFGERAR_MULTIPLICADOR = 'MULTIPLICADOR';
    const IDREFGERAR_DIVISOR = 'DIVISOR';
    const IDREFGERAR_NENHUM = 'NENHUM';
    const IDRETREFERENCIA_NAO = 'N';
    const IDRETREFERENCIA_SIM = 'S';
    
    public function __construct($entityManager, $hcmParameters) {
        $this->hcmParameters = $hcmParameters;
        parent::__construct(null,$entityManager);
    }

    private function _inicializaLista($nrOrg, $nrOrgPadrao, $nrGrupoIncidencia) {

        $itensGrupo = $this->entityManager->getRepository(Repositories::FPA_GRPITEM)->findBy(array(
            'nrorg' => array($nrOrg,$nrOrgPadrao),'nrgrpincide' => $nrGrupoIncidencia), array('nreventom' => 'DESC'));
            
        foreach ($itensGrupo as $item) {
            $this->addItem($item, $item->getNritem());
        }
    }

    public function carregaItensGrupoPorGrupoIncidencia($nrOrg, $nrOrgPadrao, $nrGrupoIncidencia) {

        if ($this->_nrGrpIncidAnterior !== $nrGrupoIncidencia) {
            $this->resetItems();
            $this->_inicializaLista($nrOrg, $nrOrgPadrao, $nrGrupoIncidencia);
            $this->_nrGrpIncidAnterior = $nrGrupoIncidencia;
        } else {
            if ($this->countItems() === 0) {
                $this->_inicializaLista($nrOrg, $nrOrgPadrao, $nrGrupoIncidencia);
            }
        }
    }

    public function getTipoIncidencia($nrItem){
        $itemGrupo = $this->getItem($nrItem);
        if (is_object($itemGrupo) === true){
            $nrTipoIncidencia = $itemGrupo->getNrtipoincide();
            return $nrTipoIncidencia;
        } else {
            throw new \Exception('Tipo de incidencia nao parametrizado para o item '.$nrItem.'.');
            return null;
        }
    }


    public function calculaValorItemGrupoIncidencia($mesCompetencia, $itemCalculo, $evento, $nrItem){
        $valor = 0;
        $nrTipoIncidencia = $this->getTipoIncidencia($nrItem);
        if ($nrTipoIncidencia !== null){
            $valor = $itemCalculo->calculaIncidencias($nrTipoIncidencia, $mesCompetencia, $evento);
        }
        return $valor;
    }

    public function calculaReferenciaItemGrupoIncidencia($mesCompetencia, $itemCalculo, $evento, $nrItem){
        $referencia = 0;
        $nrTipoIncidencia = $this->getTipoIncidencia($nrItem);
        $referencia = $itemCalculo->calculaReferenciaIncidencias($nrTipoIncidencia, $mesCompetencia, $evento);
        return $referencia;
    }

    public function getListaIncidencias(){
        $listaTipoIncidencia = array();
        $itens = $this->getItems();
        foreach ($itens as $item){
            $nrItem = $item->getNritem();
            $listaTipoIncidencia[] = $this->getTipoIncidencia($nrItem);
        }

        return $listaTipoIncidencia;
    }

}

