<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\MsgCalculo;
use HCM\Util\TkEntity;
use HCM\FPA\Entity\FpaincideventoEntity;
use HCM\Util\DateUtil;

class FpaitecalcestrutEntity extends TkEntity
{

    const MAIS = '+'; /* Indica somatorio de valor */
    const MENOS = '-'; /* Indica diferenca de valor */

    private $menorDtOcorr;
    private $maiorDtOcorr;
    
    private $nrestruturam;
    private $nrcalculofolha;
    private $nrorg;
    private $nrorgPadrao;

    public function __construct($entityManager, $hcmParameters) {
        
        $this->incideventoEntity = new FpaincideventoEntity($entityManager, $hcmParameters); 
        $this->hcmParameters = $hcmParameters;
        parent::__construct(null, $entityManager);
    }
    
    public function retornaItensPorEstrutura($nr_org, $nr_estrutura, $nr_calculo){

        if ($nr_calculo === null) {
            MsgCalculo::getInstance()->adicionaMensagem('Numero de calculo nulo.',MsgCalculo::ERRO,true);
        }

        $this->resetItems();
        $contador = 0;

        $this->nrestruturam = $nr_estrutura;
        $this->nrcalculofolha = $nr_calculo;
        $this->nrorg = $nr_org;
        $this->nrorgPadrao = $this->hcmParameters->getNrorgPadrao();

        $resultado = $this->entityManager->getRepository(Repositories::FPA_ITECALCESTRUT)-> retornaItem($nr_estrutura, $nr_calculo, $nr_org);

        if (empty($resultado) === false) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj);
                $contador++;
            }
        }
        return $contador;
    }

    public function calculaIncidencias($tipo_incidencia, $data_competencia, $eventom,
                                       $referenciaOcorrencia = false)
    {
        $base = 0.0;
        $bases = array();
        
        foreach ($this->getItems() as $item) {
            $evento = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->findOneBy(array('nrorg' => array($this->nrorg,$this->nrorgPadrao),'nreventom' => $item->getNreventom()));
            if (is_object($evento) === true) {
                $lista_id_aux_incide_evento = $this->incideventoEntity->incide(
                    $tipo_incidencia, $evento->getNreventom(), $data_competencia
                );
                foreach($lista_id_aux_incide_evento as $nrOrg => $aux_incide_evento) {
                    if ($aux_incide_evento === self::MAIS) {
                        $base += $item->getVrlancestrutura();
                    } elseif ($aux_incide_evento === self::MENOS) {
                        $base -= $item->getVrlancestrutura();
                    }

                    if (is_null($this->menorDtOcorr)) {
                        $this->menorDtOcorr = DateUtil::getDataDeString($item->getDtocorrencia());
                    } else {
                        $menorData = DateUtil::getDataDeString($item->getDtocorrencia());
                        $this->menorDtOcorr = DateUtil::retornaMenorData($this->menorDtOcorr, $menorData);
                    }
                    if (is_null($this->maiorDtOcorr)) {
                        $this->maiorDtOcorr = DateUtil::getDataDeString($item->getDtocorrencia());
                    } else {
                        $maiorData = DateUtil::getDataDeString($item->getDtocorrencia());
                        $this->maiorDtOcorr = DateUtil::retornaMaiorData($this->maiorDtOcorr, $maiorData);
                    }
                    
                    if ($referenciaOcorrencia && $aux_incide_evento === self::MAIS) {
                        $dtocorrencia = DateUtil::getDataDeString(substr($item->getDtocorrencia(), 0, 10),DateUtil::FORMATO_BRASILEIRO, true);
                        $bases[$dtocorrencia->getTimestamp()] = isset($bases[$dtocorrencia->getTimestamp()]) ?
                            $bases[$dtocorrencia->getTimestamp()] + $item->getVreventofolha() : $item->getVreventofolha();
                    }elseif ($referenciaOcorrencia && $aux_incide_evento === self::MENOS) {
                        $dtocorrencia = DateUtil::getDataDeString(substr($item->getDtocorrencia(), 0, 10),DateUtil::FORMATO_BRASILEIRO, true);
                        $bases[$dtocorrencia->getTimestamp()] = isset($bases[$dtocorrencia->getTimestamp()]) ?
                            $bases[$dtocorrencia->getTimestamp()] - $item->getVreventofolha() : $item->getVreventofolha();
                    }
                }
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Calculo de Incidencias: Evento '.$item->getNreventom().' não encontrado para a Organização');
            }
        }
        return ($referenciaOcorrencia) ? $bases : $base;
    }

    public function getMenorDtOcorrencia()
    {
        return $this->menorDtOcorr;
    }

    public function getMaiorDtOcorrencia()
    {
        return $this->maiorDtOcorr;
    }
}