<?php

namespace HCM\FPA\Registro\sefip;

use HCM\FPA\Registro\RegistroSEFIP;

class Registro21SEFIP extends RegistroSEFIP{


    const REGISTRO_21 = '21';

    const CARACTER_HIFEN = ' - ';

    private $_estrutura;

    private $_tpInscricao;
    private $_nrInscricao;
    private $_tpInscricaoTomador;
    private $_nrInscricaoTomador;
    private $_nrRecolhimento;
    private $_fpas;
    private $_valorSalarioFamilia;

    private $_valoresEstrutura;


    /**
     * Gera Tipo de Registro 21.
     *
     * @param EstruturamEntity $estrutura          Instancia de EstruturamEntity.
     * @param EnderecoEntity   $endereco           Instancia de EnderecoEntity.
     *
     * @return void
     *
     * @author felipesantos - 12/07/2017
     */
    public function geraRegistro21($estrutura){
        $this->_estrutura = $estrutura;

        $this->_nrRecolhimento = $this->getCodigoRecolhimento();
        $this->_fpas = $this->_estrutura->historico->getCdfpas();
        $this->_geraRegistro();
    }


    /**
     * Atribui os dados no atributo da classe Pai.
     *
     * @return void
     *
     * @author alessandrobartels - 04/01/2012
     */
    private function _geraRegistro(){
        // Bloco 1 ao 3 eh gerado no pai.
        parent::geraRegistro(self::REGISTRO_21, $this->_tpInscricao, $this->_nrInscricao);

        // Bloco 4 | 018 | Tipo de Inscricao do tomador.
        $this->atribuiDado($this->_tpInscricaoTomador, 1);

        // Bloco 5 | 019-032 | Inscricao do tomador.
        $this->atribuiDado($this->_nrInscricaoTomador, 14, parent::CARACTER_ZERO, self::PREENCHE_ESQUERDA);

        // Bloco 6 | 033-053 | Preencher com zeros.
        $this->atribuiDado(parent::CARACTER_ZERO, 21, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 7 | 054-068 | Compensacao - Valor Corrigido
        $this->_atribuiCompensacao($this->_valoresEstrutura[RegistroSEFIP::SEFIP_COMPEN_VLRCORR]);

        // Bloco 8 | 069-074 | Compensacao - Periodo Inicio
        $this->_atribuiCompensacaoPerIni($this->_valoresEstrutura['DTINICIAL'], $this->_valoresEstrutura[RegistroSEFIP::SEFIP_COMPEN_VLRCORR]);

        // Bloco 9 | 075-080 | Compensacao - Periodo Fim
        $this->_atribuiCompensacaoPerFim($this->_valoresEstrutura['DTFINAL'], $this->_valoresEstrutura[RegistroSEFIP::SEFIP_COMPEN_VLRCORR]);

        // Bloco 10 | 081-095 | Recolhimento de Competências Anteriores - Valor do INSS sobre folha de pagamento
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 11 | 096-110 | Recolhimento de Competências Anteriores - Outras Entidades sobre folha de pagamento
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 12 | 111-125 | Parcelamento do FGTS – somatório das remunerações das categorias 01, 02, 03, 05 e 06
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 13 | 126-140 | Parcelamento do FGTS – somatório das remunerações das categorias 04 e 07
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 14 | 141-155 | Parcelamento do FGTS - valor recolhido
        $this->atribuiDado(parent::CARACTER_ZERO, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);

        // Bloco 15 | 156-359 | Brancos.
        $this->atribuiDado(parent::CARACTER_VAZIO, 204, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);

        // Bloco 22 | 360 | Final de linha. Inserir '*'.
        $this->atribuiDado(parent::CARACTER_VAZIO, 1, parent::CARACTER_ASTERISCO);
    }


    public function setNrInscricao($nrInscricao)
    {
        $this->_nrInscricao = $nrInscricao;
    }


    public function setTpInscricao($tpInscricao)
    {
        $this->_tpInscricao = $tpInscricao;
    }


    public function setNrInscricaoTomador($nrInscricaoTomador)
    {
        $this->_nrInscricaoTomador = $nrInscricaoTomador;
    }


    public function setTpInscricaoTomador($tpInscricaoTomador)
    {
        $this->_tpInscricaoTomador = $tpInscricaoTomador;
    }

    public function setValoresEstrutura($dadoEstrutura)
    {
        $this->_valoresEstrutura = $dadoEstrutura;
    }


    /**
     * Atribui Compensação - Valor Corrigido.
     *
     * Campo opcional para os códigos de recolhimento 130, 135, 150, 155, 211 e 608.
     * Não pode ser informado para os código de recolhimento 317 e 337.
     * Não pode ser informado na competência 13.
     * Só deve ser informado se o Indicador de Recolhimento da Previdência Social (campo 20 do registro 00) for igual a 1 (GPS no prazo).
     * Não pode ser informado para competência anterior a outubro de 1998.
     * Sempre que não informado preencher com zeros.
     *
     * @param number $vrCompensacao Valor de Compensação - Valor Corrigido.
     *
     * @return void
     *
     * @author felipesantos - 05/05/2015
     */
    private function _atribuiCompensacao($vrCompensacao){

        $codRecol = array(
            '317', '337'
        );
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false)) {
            $vrCompensacao = parent::CARACTER_ZERO;
        }

        if ($this->getCompetencia13()) {
            $vrCompensacao = parent::CARACTER_ZERO;
        }

        if ($this->getIdRecolhimentoGPS() !== parent::IND_REC_GPS_NOPRAZO) {
            $vrCompensacao = parent::CARACTER_ZERO;
        }

        $this->atribuiDado($vrCompensacao, 15, parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui Compensação - Período Início.
     *
     * @param string $dataInicio Compensação - Período Início (AAAA/MM).
     * @param number $vrCompensacao Valor de Compensação - Valor Corrigido.
     *
     * @return void
     *
     * @author felipesantos - 05/05/2015
     */
    private function _atribuiCompensacaoPerIni($dataInicio, $vrCompensacao){

        if ($vrCompensacao == 0) {
            $dataInicio = parent::CARACTER_BRANCO;
        }

        $codRecol = array(
            '317', '337'
        );
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false)) {
            $dataInicio = parent::CARACTER_BRANCO;
        }

        if ($this->getCompetencia13()) {
            $dataInicio = parent::CARACTER_BRANCO;
        }

        if ($this->getIdRecolhimentoGPS() !== parent::IND_REC_GPS_NOPRAZO) {
            $dataInicio = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataInicio, 6, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
    }


    /**
     * Atribui Compensação - Período Fim.
     *
     * @param string $dataFim Compensação - Período Fim (AAAA/MM).
     * @param number $vrCompensacao Valor de Compensação - Valor Corrigido.
     *
     * @return void
     *
     * @author felipesantos - 05/05/2015
     */
    private function _atribuiCompensacaoPerFim($dataFim, $vrCompensacao){

        if ($vrCompensacao == 0) {
            $dataFim = parent::CARACTER_BRANCO;
        }

        $codRecol = array(
            '317', '337'
        );
        if ((array_search($this->_nrRecolhimento, $codRecol) !== false)) {
            $dataFim = parent::CARACTER_BRANCO;
        }

        if ($this->getCompetencia13()) {
            $dataInicio = parent::CARACTER_BRANCO;
        }

        if ($this->getIdRecolhimentoGPS() !== parent::IND_REC_GPS_NOPRAZO) {
            $dataInicio = parent::CARACTER_BRANCO;
        }

        $this->atribuiDado($dataFim, 6, parent::CARACTER_BRANCO, parent::PREENCHE_ESQUERDA);
    }


    public function alteraDadoRegistro21($registroMag, $dado) {

        // Salario Familia - Posicao Inicial = 198 Tamanho = 15. Subtrai 1 da posicao inicial, pois o array inicia em 0.
        $linha = $this->alteraDado($registroMag->getDslinharegistro(), $dado[self::SEFIP_SALARIO_FAMILIA], (198 - 1), 15,
            parent::CARACTER_ZERO, parent::PREENCHE_ESQUERDA);
        $registroMag->setDslinharegistro($linha);
        $registroMag->setVrsalfamilia($dado[self::SEFIP_SALARIO_FAMILIA]);
        $this->_valorSalarioFamilia = $registroMag->getVrsalfamilia();

    }

}