<?php
/**
 * Created by PhpStorm.
 * User: felipesantos
 * Date: 18/08/2017
 * Time: 11:09
 */

namespace HCM\Geral\Controllers;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Logic\ESocialLogic;
use HCM\Geral\Services\ESocialService;
use HCM\Geral\Services\EsoSimplificadoService;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class EsoSimplificadoController extends Crud {

    protected $dataSourceName = '';

    protected $dtsGetEsonaoperiodico = 'getEsonaoperiodico';
    protected $dtsSetEsonaoperiodico = 'setEsonaoperiodico';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_NRTPAMBIENTE', 'P_NRESTRUTURAM');
    
    // Array que armazena as mensagens geradas na rotina 'processEnvio'. É organizada da seguinte forma: 'nomeMensagem' => 'valoresAssociados'
    protected $mensagensProcessEnvio = array();

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, EntityManager $entityManager, HCMParameters $hcmParameters, 
                                 EsoSimplificadoService $esoSimplificadoService, ESocialService $eSocialService){
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->esoSimplificadoService = $esoSimplificadoService;
        $this->eSocialService = $eSocialService;
        
        parent::__construct($dataSourceManager);
    }
    
    public function getEsonaoperiodico(DTO\Request\Filter $request, DTO\Response $response){
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dtsGetEsonaoperiodico);
        $filterData->setPageSize(99999);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }

    public function setEsonaoperiodico(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            
            $this->dataSourceName = $this->dtsSetEsonaoperiodico;
            
            foreach ($request->getDataSet()->getRows() as &$row) {
                if ($row['__is_new']) {
                    $row['NRORG'] = $this->hcmParameters->getNrorgTrabalho();
                    
                    $row['IDATIVO'] = isset($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
                    $row['IDSTATUS'] = isset($row['IDSTATUS']) ? $row['IDSTATUS'] : 'P';
                    $row['IDEVENTO'] = isset($row['IDEVENTO']) ? $row['IDEVENTO'] : null;
                    $row['DTENVIO'] = isset($row['DTENVIO']) ? $row['DTENVIO'] : null;
                    $row['DTRESPOSTA'] = isset($row['DTRESPOSTA']) ? $row['DTRESPOSTA'] : null;
                    $row['DTPROCESSAMENTO'] = isset($row['DTPROCESSAMENTO']) ? $row['DTPROCESSAMENTO'] : null;
                    $row['NRPROTOCOLO'] = isset($row['NRPROTOCOLO']) ? $row['NRPROTOCOLO'] : null;
                    $row['NRRECIBO'] = isset($row['NRRECIBO']) ? $row['NRRECIBO'] : null;
                    $row['CDERRO'] = isset($row['CDERRO']) ? $row['CDERRO'] : null;
                    $row['DSDIRXMLENV'] = isset($row['DSDIRXMLENV']) ? $row['DSDIRXMLENV'] : null;
                    $row['DSDIRXMLRETENV'] = isset($row['DSDIRXMLRETENV']) ? $row['DSDIRXMLRETENV'] : null;
                    $row['DSDIRXMLRETPRO'] = isset($row['DSDIRXMLRETPRO']) ? $row['DSDIRXMLRETPRO'] : null;
                    $row['DSDIRXMLRETTOT'] = isset($row['DSDIRXMLRETTOT']) ? $row['DSDIRXMLRETTOT'] : null;
                    $row['DSOBSERVACAO'] = isset($row['DSOBSERVACAO']) ? $row['DSOBSERVACAO'] : null;
                    $row['DSOBSERVACAO2'] = isset($row['DSOBSERVACAO2']) ? $row['DSOBSERVACAO2'] : null;
                    $row['NRENVIOEXCL'] = isset($row['NRENVIOEXCL']) ? $row['NRENVIOEXCL'] : null;
                    $row['VRBASEMENSAL'] = isset($row['VRBASEMENSAL']) ? $row['VRBASEMENSAL']: null;
                    $row['VRBASE13'] = isset($row['VRBASE13']) ? $row['VRBASE13']: null;
                    $row['VRVALORMENSAL'] = isset($row['VRVALORMENSAL']) ? $row['VRVALORMENSAL']: null;
                    $row['VRVALOR13'] = isset($row['VRVALOR13']) ? $row['VRVALOR13']: null;
                    $row['VRVALORMATERMENSAL'] = isset($row['VRVALORMATERMENSAL']) ? $row['VRVALORMATERMENSAL']: null;
                    $row['VRVALORMATER13'] = isset($row['VRVALORMATER13']) ? $row['VRVALORMATER13']: null;
                    $row['VRSALFAMILIA'] = isset($row['VRSALFAMILIA']) ? $row['VRSALFAMILIA']: null;
                    $row['VRSALFAMILIA13'] = isset($row['VRSALFAMILIA13']) ? $row['VRSALFAMILIA13']: null;
                    $row['VRCALCULADO'] = isset($row['VRCALCULADO']) ? $row['VRCALCULADO']: null;
                    $row['VRDESCONTADO'] = isset($row['VRDESCONTADO']) ? $row['VRDESCONTADO']: null;
                    $row['NRESTRUTLOTACAO'] = isset($row['NRESTRUTLOTACAO']) ? $row['NRESTRUTLOTACAO']: null;
                    
                    $arrItens = $this->eSocialService->getItensParaEnvioEvento($row['NMEVENTO'], $row['IDGRUPO'], $row);
                    if (empty($arrItens)) {
                        $this->addMensagemProcessEnvio(Messages::HCM09000_ESO_ENVIOS_NOT_FOUND_ERROR, $row['NMEVENTO']);
                        break;
                    }
                    
                    foreach($arrItens as $item) {
                        $row['NRCHAVE'] = $item['NRCHAVE'];
                        $row['NRTABELA'] = isset($item['NRTABELA']) ? $item['NRTABELA'] : '';
                        $row['IDOPERACAO'] = isset($item['IDOPERACAO']) ? $item['IDOPERACAO'] : 'I';
                        $row['DTMESCOMPETENC'] = $item['DTMESCOMPETENC'];
                        $row['NRVINCULOM'] = $item['NRVINCULOM'];
            
                        try {
                            $this->eSocialService->beforeInsertEnvioCadastros($row['NRESTRUTURAM'], $row['NRTPAMBIENTE'], $row['NREVENTO'], $row['NRCHAVE'], $row['NRTABELA'], $row['IDOPERACAO'], $row['DTMESCOMPETENC']);
                        } catch(\Exception $e) {
                            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE) {
                                if($e->getMessage() == Messages::HCM09000_ESO_UNIQUE_ENVIO_ERROR){
                                    $this->addMensagemProcessEnvio(Messages::HCM09000_ESO_UNIQUE_ENVIO_ERROR, $row['NMEVENTO']);
                                } else {    
                                    $response->addMessage(new DTO\Response\Message($e->getMessage()), DTO\Response\Message::TYPE_WARNING);
                                }
                                continue;
                            } else {
                                throw $e;
                            }
                        }
                        
                        if (array_key_exists('NRENVIO', $row)) {
                            if ($row['NRENVIO'] === '') {
                                $row['NRENVIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_NAOPERIODICO', 1, 12, true);
                            }
                        } else {
                            $row['NRENVIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_NAOPERIODICO', 1, 12, true);
                        }
                        
                        parent::save($request, $response);
                    }
                } else {
                    parent::save($request, $response);
                }
            }
            
            if($this->existemMensagensProcessEnvio()){
                $response->addMessage(new DTO\Response\Message($this->getMensagensProcessEnvio(), DTO\Response\Message::TYPE_WARNING));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteEsonaoperiodico(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            $retorno = false;
            foreach ($request->getDataSet()->getRows() as &$row) {
                $this->esoSimplificadoService->beforeDeleteEvento($row);
                $this->eSocialService->deleteDadosEnvio($row);
                $retorno = $this->eSocialService->inativaEvento($row['NRORG'], $row['NRESTRUTURAM'], $row['NRENVIO']);
            }
            if($retorno){
                $response->addNotification(new DTO\Response\Notification('deleteEsonaoperiodico', DTO\Response\Notification::TYPE_SUCCESS));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function enviarEventoNaoPeriodico(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            
            foreach ($request->getDataSet()->getRows() as $row) {
                $logic = new ESocialLogic($this->entityManager, $this->hcmParameters, $row['NRESTRUTURAM'], $row['NRTPAMBIENTE']);
                $msgRetorno = $logic->enviarEventoNaoPeriodico($row);
                $msgValidacao = $logic->getMessages();
            }
            
            if (count($msgValidacao) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('enviarEventoNaoPeriodico', $msgValidacao));
            } elseif ($msgRetorno) {
                $response->addMessage(new DTO\Response\Message($msgRetorno));
            } else {
                $response->addMessage(new DTO\Response\Message("Envio de evento não disponível", DTO\Response\Message::TYPE_WARNING));
            }
        } catch(\Exception $e) {
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }
    
    public function consultarProtocoloNaoPeriodico(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            
            foreach ($request->getDataSet()->getRows() as $row) {
                $logic = new ESocialLogic($this->entityManager, $this->hcmParameters, $row['NRESTRUTURAM'], $row['NRTPAMBIENTE']);
                $msgRetorno = $logic->consultarProtocoloNaoPeriodico($row);
                $msgValidacao = $logic->getMessages();
            }
            
            if (count($msgValidacao) > 0) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('consultarProtocoloNaoPeriodico', $msgValidacao));
            } elseif ($msgRetorno) {
                $response->addMessage(new DTO\Response\Message($msgRetorno));
            } else {
                $response->addMessage(new DTO\Response\Message("Envio de evento não disponível", DTO\Response\Message::TYPE_WARNING));
            }
        } catch(\Exception $e) {
            if ($e->getCode() == Messages::ALERT_MESSAGE_CODE || $e->getCode() == Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function getXmlEsoSimplificado(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $xmlFile = null;
            $idEvento = null;
            $dsDirXml = null;
            foreach ($request->getFilterCriteria()->getConditions() as $condition) {
                if ($condition['columnName'] == 'P_XMLFILE') {
                    $xmlFile = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_IDEVENTO') {
                    $idEvento = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_DSDIRXML') {
                    $dsDirXml = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_NRTPAMBIENTE') {
                    $nrTpAmbiente = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_IDGRUPO') {
                    $idGrupo = $this->processConditionValue($condition);
                }
                if ($condition['columnName'] == 'P_CDCNPJESTRUT') {
                    $cdCnpjEstrut = $this->processConditionValue($condition);
                }
            }
            // Alteração temporária para ler os arquivos no servidor AWS
            $dsDirXmlFilesPath = 
                ($nrTpAmbiente == 1 ? 'Producao' : 'ProducaoRestrita') . "/" .
                ($idGrupo == 'P' ? 'Periodicos' : ($idGrupo == 'N' ? 'NaoPeriodicos' : 'Cadastros')) . "/" .
                $cdCnpjEstrut . "/" .
                ($xmlFile == 'envio' ? 'Envio' : ($xmlFile == 'retorno' ? 'RetornoEnvio' : ($xmlFile == 'consulta' ? 'RetornoProcessamento' : ($xmlFile == 'total' ? 'Totalizador' : '')))). "/" .
                $dsDirXml;
            $xml = $this->eSocialService->getXmlFiles($nrorg, $xmlFile, $idEvento, $dsDirXml, $dsDirXmlFilesPath);
            
            // -------
            $xmlDataSet = $this->eSocialService->setXmlFileToDataSet($xml);
            $dataSet = new DataSet($request->getRoutePath(), $xmlDataSet);
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->addMessage(new DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($dsDirXml, $e->getCode()));
        }
    }
    
    public function esvaziaMensagensProcessEnvio () {
        $this->mensagensProcessEnvio = array();
    }
    
    public function addMensagemProcessEnvio ($mensagem, $valor) {
        $mensagensExistentesArray = array_keys($this->mensagensProcessEnvio);
        
        if(!in_array($mensagem, $mensagensExistentesArray)){
            $this->mensagensProcessEnvio[$mensagem] = array($valor);
        } else {
            if(!in_array($valor, $this->mensagensProcessEnvio[$mensagem])){
                array_push($this->mensagensProcessEnvio[$mensagem], $valor);
            }
        }
    }
    
    public function existemMensagensProcessEnvio ($mensagem = null) {
        if (!empty($mensagem)) {
            return array_key_exists($mensagem, $this->mensagensProcessEnvio) ? !empty($this->mensagensProcessEnvio[$mensagem]) : false;
        } else {
            return count($this->mensagensProcessEnvio) > 0;
        }
    }
    
    /**
     * Retorna as mensagens salvas no array 'mensagensProcessEnvio' da seguinte maneira: mensagem1. (valor1, valor2. ...). mensagem2. (valor1, valor2. ...)
     */
    public function getMensagensProcessEnvio () {
        if ($this->existemMensagensProcessEnvio()) {
            $mensagemFinal = "";
            foreach ($this->mensagensProcessEnvio as $chave => $mensagens) {
                $mensagemFinal .= $chave;
                if(is_array($mensagens)){
                    $mensagemValores = "(";
                    foreach ($mensagens as $valor) {
                        $mensagemValores .= $valor . ", ";
                    }
                    $mensagemValores = substr($mensagemValores, 0 , -2);
                    $mensagemValores .= ")";
                    $mensagemFinal .= " " . $mensagemValores;
                }
                $mensagemFinal .= ". ";
            }
            return $mensagemFinal;
        } else {
            return false;
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}