<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;

class ParametrosOrganizacaoController extends Crud {
    
    protected $dataSourceName  = 'gpe_paramOrg_save';
    protected $dataSourceName2 = 'gpe_paramOrg';

    protected $parameterBagColumns = array('NRORG');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters   = $hcmParameters;
        parent::__construct($dataSourceManager);
    }


    public function getParamOrg (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName2);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            $row['NRPERFACESSADM'] = explode(",", $row['NRPERFACESSADM']);
            foreach($row['NRPERFACESSADM'] as $chave => $value)
                $row['NRPERFACESSADM'][$chave] = intval($value);
            
            $row['NMPERFACESSADM'] = explode(",", $row['NMPERFACESSADM']);
            
            $row['NRCANALCOMUNICACAO'] = explode(",", $row['NRCANALCOMUNICACAO']);
            foreach($row['NRCANALCOMUNICACAO'] as $chave => $value)
                $row['NRCANALCOMUNICACAO'][$chave] = intval($value);
            
            $row['NMCANALCOMUNICACAO'] = explode(",", $row['NMCANALCOMUNICACAO']);
            
            $row['NRPERFACESSOPERAPP'] = explode(",", $row['NRPERFACESSOPERAPP']);
            foreach($row['NRPERFACESSOPERAPP'] as $chave => $value)
                $row['NRPERFACESSOPERAPP'][$chave] = intval($value);
            
            $row['NMPERFACESSOPERAPP'] = explode(",", $row['NMPERFACESSOPERAPP']);
            
        }
        $response->addDataSet(new DataSet($this->dataSourceName2, $rows));
    }

    
    public function saveParamOrg (DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $row['DTULTATU'] = $row['DTATUAL'];
                $row['CDOPERULTATU'] = $row['CDOPERLOG'];
                $row['NRORGULTATU'] = $row['NRORGOPERLOG'];
                $row['DSEMAILRESPBTL'] = isset($row['DSEMAILRESPBTL']) ? str_replace(" ", "", $row['DSEMAILRESPBTL']) : null;
                $row['NRPERFACESSADM'] = isset($row['NRPERFACESSADM']) ? implode(",", $row['NRPERFACESSADM']) : null;
                $row['NRCANALCOMUNICACAO'] = isset($row['NRCANALCOMUNICACAO']) ? implode(",", $row['NRCANALCOMUNICACAO']) : null;
                $row['NRPERFACESSOPERAPP'] = isset($row['NRPERFACESSOPERAPP']) ? implode(",", $row['NRPERFACESSOPERAPP']) : null;
                $row['NRTPESTRUTOMADOR'] = array_key_exists('NRTPESTRUTOMADOR', $row) ? $row['NRTPESTRUTOMADOR'] : null;
                $row['IDVALIDAHORABASEPORTAL'] = array_key_exists('IDVALIDAHORABASEPORTAL', $row) ? $row['IDVALIDAHORABASEPORTAL'] : null;
                $row['HRTEMPODESCANSODIARIO'] = array_key_exists('HRTEMPODESCANSODIARIO', $row) ? $row['HRTEMPODESCANSODIARIO'] : null;
                $row['HRTEMPOTOLERANCIAANT'] = array_key_exists('HRTEMPOTOLERANCIAANT', $row) ? $row['HRTEMPOTOLERANCIAANT'] : null;
                $row['HRTEMPOTOLERANCIAPOS'] = array_key_exists('HRTEMPOTOLERANCIAPOS', $row) ? $row['HRTEMPOTOLERANCIAPOS'] : null;
                $row['HRTEMPOINTERVALOIGNORADO'] = array_key_exists('HRTEMPOINTERVALOIGNORADO', $row) ? $row['HRTEMPOINTERVALOIGNORADO'] : null;
                $row['IDREALIZACONTAB'] = array_key_exists('IDREALIZACONTAB', $row) ? $row['IDREALIZACONTAB'] : null;
                $row['IDREALIZADESPESA'] = array_key_exists('IDREALIZADESPESA', $row) ? $row['IDREALIZADESPESA'] : null;
                $row['DSHOSTSMTP'] = array_key_exists('DSHOSTSMTP', $row) ? $row['DSHOSTSMTP'] : null;
                $row['DSPORTASMTP'] = array_key_exists('DSPORTASMTP', $row) ? $row['DSPORTASMTP'] : null;
                $row['DSEMAILSMTP'] = array_key_exists('DSEMAILSMTP', $row) ? $row['DSEMAILSMTP'] : null;
                $row['DSSENHASMTP'] = array_key_exists('DSSENHASMTP', $row) ? $row['DSSENHASMTP'] : null;
                $row['IDSMTPSECURE'] = array_key_exists('IDSMTPSECURE', $row) ? $row['IDSMTPSECURE'] : null;
                $row['NRMENSANIVERSARIO'] = array_key_exists('NRMENSANIVERSARIO', $row) ? $row['NRMENSANIVERSARIO'] : null;
                $row['NRMENSTEMPOEMPRESA'] = array_key_exists('NRMENSTEMPOEMPRESA', $row) ? $row['NRMENSTEMPOEMPRESA'] : null;
                $row['DTCOMPETENCIA'] = array_key_exists('DTCOMPETENCIA', $row) ? $row['DTCOMPETENCIA'] : $this->hcmParameters->getDtCompetencia();
                $row['NRORGPADRAO'] = array_key_exists('NRORGPADRAO', $row) ? $row['NRORGPADRAO'] : $this->hcmParameters->getNrorgPadrao();
                $row['NRORG'] = array_key_exists('NRORG', $row) ? $row['NRORG'] : $this->hcmParameters->getNrorgTrabalho();
                parent::save($request, $response);
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }

}