<?php

namespace HCM\Geral\Logic\Esocial;

use HCM\Geral\Factory\ESocialCadIniFactory;
use HCM\Geral\Factory\ESocialNaoPerFactory;
use HCM\Geral\Factory\ESocialPeriodFactory;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\MsgProcesso;
use HCM\Util\Repositories;

class ESocialDataFactory {

    protected $entityManager;
    protected $hcmParameters;
    
    private $_dsVersao;
    private $_projects;
    private $_helpers;

    private $_methodGetDados;
    
    private $_nmEvento;
    private $_idOperacao;
    private $_idEvento;
    
    private $tagsTrad;

    public function __construct($entityManager, $hcmParameters, $nmEvento, $dsVersao, $tpInscEmpregador, $nrInscEmpregador)
    {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        
        $this->_procEmi = 1;
        $this->_verProc = $this->hcmParameters->getProjectVersion();
        
        $this->_dsVersao = $dsVersao;
        
        $this->_datasourcePath = ESOCIAL_FILES_PATH . DIRECTORY_SEPARATOR . "datasource";
        
        $projectsPath = $this->_datasourcePath . "/projects_v_$dsVersao/";
        $helpersPath = $this->_datasourcePath . "/helpers_v_$dsVersao/";
        
        if (file_exists($projectsPath . $nmEvento . ".json")) {
            $this->_projects = json_decode(file_get_contents($projectsPath . $nmEvento . ".json"), true);
        } else {
            throw new \Exception("Não foi encontrado o arquivo $nmEvento.json");
        }
        
        if (file_exists($helpersPath . $nmEvento . ".json")) {
            $this->_helpers = json_decode(file_get_contents($helpersPath . $nmEvento . ".json"), true);
        } else {
            $this->_helpers = null;
        }

        $this->_methodGetDados = 'getDados';
        
        $this->_nmEvento = str_replace('-','',$nmEvento);
        
        $this->_tpInscEmpregador = $tpInscEmpregador;
        $this->_nrInscEmpregador = $nrInscEmpregador;
        
        $this->tagsTrad = array();
    }


    /*-----------------------------------------------------------------------------------------------------------------*/

    private function getOperacaoEvento($idOperacao) {
        if ($idOperacao == 'I') {
            return 'inclusao';
        }
        if ($idOperacao == 'A') {
            return 'alteracao';
        }
        if ($idOperacao == 'E') {
            return 'exclusao';
        }
        return null;
    }

    private function setOperacaoEvento(array $dadosEvento, $idOperacao) {
        if ($idOperacao == 'I') {
            $dadosEvento['INCLUSAO'] = '__inclusao__';
        }
        if ($idOperacao == 'A') {
            $dadosEvento['ALTERACAO'] = '__alteracao__';
        }
        if ($idOperacao == 'E') {
            $dadosEvento['EXCLUSAO'] = '__exclusao__';
        }
        return $dadosEvento;
    }
    
    /**
     * Método chamado dinamicamente pelo método buildDadosXml
     */
    private function getDadosConsultaIdentificadoresEventosEmpregador($dados) {
        return array(
            'TPINSC' => $this->_tpInscEmpregador,
            'NRINSC' => $this->_nrInscEmpregador,
            'TPEVT' => $dados['TPEVT'],
            'PERAPUR' => str_replace('/','-',$dados['PERAPUR']),
        );
    }
    
    /**
     * Método chamado dinamicamente pelo método buildDadosXml
     */
    private function getDadosConsultaIdentificadoresEventosTabela($dados) {
        $dtIni = ($dados['DTINI'] ? DateUtil::getDataDeString($dados['DTINI'], DateUtil::FORMATO_BRASILEIRO_DATAHORA) : null);
        $dtFim = ($dados['DTFIM'] ? DateUtil::getDataDeString($dados['DTFIM'], DateUtil::FORMATO_BRASILEIRO_DATAHORA) : null);
        return array(
            'TPINSC' => $this->_tpInscEmpregador,
            'NRINSC' => $this->_nrInscEmpregador,
            'TPEVT' => $dados['TPEVT'],
            'CHEVT' => $dados['CHEVT'],
            'DTINI' => ($dtIni ? $dtIni->format('Y-m-d') . 'T' . $dtIni->format('H:i:s') : null),
            'DTFIM' => ($dtIni ? $dtFim->format('Y-m-d') . 'T' . $dtFim->format('H:i:s') : null),
        );
    }
    
    /**
     * Método chamado dinamicamente pelo método buildDadosXml
     */
    private function getDadosConsultaIdentificadoresEventosTrabalhador($dados) {
        $dtIni = ($dados['DTINI'] ? DateUtil::getDataDeString($dados['DTINI'], DateUtil::FORMATO_BRASILEIRO_DATAHORA) : null);
        $dtFim = ($dados['DTFIM'] ? DateUtil::getDataDeString($dados['DTFIM'], DateUtil::FORMATO_BRASILEIRO_DATAHORA) : null);
        return array(
            'TPINSC' => $this->_tpInscEmpregador,
            'NRINSC' => $this->_nrInscEmpregador,
            'CPFTRAB' => $dados['CPFTRAB'],
            'DTINI' => ($dtIni ? $dtIni->format('Y-m-d') . 'T' . $dtIni->format('H:i:s') : null),
            'DTFIM' => ($dtIni ? $dtFim->format('Y-m-d') . 'T' . $dtFim->format('H:i:s') : null),
        );
    }
    
    /**
     * Método chamado dinamicamente pelo método buildDadosXml
     */
    private function getDadosSolicitacaoDownloadEventosPorId($dados) {
        $ideventos = array();
        
        foreach($dados as $dado) {
            $ideventos[] = $dado['IDEVENTO'];
        }
        
        return array(
            'TPINSC' => $this->_tpInscEmpregador,
            'NRINSC' => $this->_nrInscEmpregador,
            'ID' => $ideventos
        );
    }
    
    /**
     * Método chamado dinamicamente pelo método buildDadosXml
     */
    private function getDadosSolicitacaoDownloadEventosPorNrRecibo($dados) {
        $recibos = array();
        
        foreach($dados as $dado) {
            $recibos[] = $dado['NRRECIBO'];
        }
        
        return array(
            'TPINSC' => $this->_tpInscEmpregador,
            'NRINSC' => $this->_nrInscEmpregador,
            'NRREC' => $recibos
        );
    }
    
    private function getFactory() {
        $factoryClass = "\HCM\Geral\Repository\Esocial\\" . $this->_nmEvento . "Repository";
        
        $factory = new $factoryClass($this->entityManager, $this->hcmParameters);
        
        return $factory;
    }
    
    /**
     * Gera ID para o evento
     * - Tamanho deve ser igual a 36 caracteres
     * - Deve ser iniciado por 'ID'
     * - O terceiro caracter deve ser 1 (CNPJ) ou 2 (CPF)
     * - CNPJ do empregador. Raíz do CNPJ ou CNPJ completo dependendo da Natureza Jurídica
     * - Data Hora da geração
     * - Sequencial de 5 dígitos com o final do NRENVIO
     * 
     * - Verifica se já existe outro envio com o mesmo ID gerado
     */
    private function geraIdEvento($nrestruturam, $nrenvio) {
        try {
            //Busca o empregador para pegar o CNPJ e a Natureza Jurídica
            $estruturah = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->findOneBy(
                array('nrestruturam' => $nrestruturam, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()),
                array('dtmescompetenc' => 'desc')
            );
            //CNPJ do empregador sem caracteres especiais
            $cdcnpjestrut = str_replace(array('.','/','-'), '', $estruturah->getCdcnpjestrut());
            
            $tipo = strlen($cdcnpjestrut) == 14 ? '1' : '2';
            if ($tipo == '1') {
                $inscricao = array_search($estruturah->getCdnatujuri(), array('1015','1040','1074','1163')) === false ? str_pad(substr($cdcnpjestrut, 0, 8), 14, '0', STR_PAD_RIGHT) : $cdcnpjestrut;
            } else {
                $inscricao = str_pad($cdcnpjestrut, 14, '0', STR_PAD_RIGHT);
            }
            $data = DateUtil::getDataAtual()->format('YmdHis');
            $seq = str_pad(substr($nrenvio, -5), 5, '0', STR_PAD_LEFT);
            
            //Monta o ID do evento: 'ID' + Tipo de inscrição + Inscrição + Data/Hora + Sequencial
            $idEvento = 'ID' . $tipo . $inscricao . $data . $seq;
            
            //Valida a formação do ID
            if (strlen($idEvento) != 36 || substr($idEvento,0,2) != 'ID' || (substr($idEvento,2,1) != '1' && substr($idEvento,2,1) != '2') || 
                (strlen(str_replace(array('.','/','-'), '', substr($idEvento,3,14))) != 11 && strlen(str_replace(array('.','/','-'), '', substr($idEvento,3,14))) != 14)) {
                throw new \Exception(Messages::HCM09000_ESO_IDEVENTO_INVALID_ERROR,Messages::ERROR_MESSAGE_CODE);
            }
            
            // Verifica se já existe outro envio com o mesmo ID
            /** @var \Teknisa\Model\Gestaopessoas\Entities\EsoEnviocadastros $envio */
            $envios = $this->entityManager->getRepository(Repositories::ESO_ENVIOCADASTROS)->findBy(array(
                'idevento' => $idEvento, 'nrorg' => $this->hcmParameters->getNrorgTrabalho()
            ));
            if (count($envios) > 0) {
                throw new \Exception(Messages::HCM09000_ESO_IDEVENTO_INVALID_ERROR,Messages::ERROR_MESSAGE_CODE);
            }
            return $idEvento;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Monta os dados do evento conforme a estrutura json e os dados retornados do repositório
     * @return array|bool
     * @throws \Exception
     */
    public function buildDadosXml($dadosEnvio, $idGrupo, $geraIdEvento, $geraVarAmb, $tagsTrad) {
        try {
            $idEvento = null;
            $dadosXml = null;
            
            $this->_idOperacao = $this->getOperacaoEvento(isset($dadosEnvio['IDOPERACAO']) ? $dadosEnvio['IDOPERACAO'] : null);
            
            $this->tagsTrad = $tagsTrad;
            
            // Método dinâmico para buscar os dados do evento (Ex: getDadosS1000)
            $method = $this->_methodGetDados . $this->_nmEvento;
            $factory = null;
            
            if ($this->_dsVersao == 'S-1-0') {
                $factory = $this->getFactory();
            } else {
                switch ($idGrupo) {
                    case 'I':
                    case 'T':
                        $factory = $this->eSocialCadIniFactory;
                        break;
                    case 'N':
                        $factory = $this->eSocialNaoPerFactory;
                        break;
                    case 'P':
                        $factory = $this->eSocialPeriodFactory;
                        break;
                    default:
                        $factory = $this;
                        break;
                }
            }
            
            if (method_exists($factory, $method)) {
                $dados = $factory->$method($dadosEnvio);
                if ($dados) {
                    /*if ($geraIdEvento) {
                        // Função para geração de ID's
                        if (is_null($dadosEnvio['IDEVENTO']) || !$dadosEnvio['IDEVENTO']) {
                            $this->idEvento = $this->geraIdEvento($dadosEnvio['NRESTRUTURAM'], $dadosEnvio['NRENVIO']);
                        } else {
                            $this->idEvento = $dadosEnvio['IDEVENTO'];
                        }
                    }*/
                    
                    // Função crucial para a query não gerar novo ID do evento caso já exista
                    if (!is_null($dadosEnvio['IDEVENTO']) && $dadosEnvio['IDEVENTO']) {
                        $dados['ID'] = $dadosEnvio['IDEVENTO'];
                    }
                    
                    if ($geraVarAmb) {
                        /* Seta as variáveis de ambiente */
                        //$dados['ID'] = $this->idEvento;
                        $dados['INDRETIF'] = ($dadosEnvio['IDOPERACAO'] == 'I' || $dadosEnvio['IDOPERACAO'] == 'A' || $dadosEnvio['IDOPERACAO'] == 'E' ? '1' : '2');
                        $dados['NRRECIBO'] = ($dados['INDRETIF'] == '2' ? $dadosEnvio['NRRECIBO2ORIG'] : NULL);
                        $dados['TPAMB'] = $dadosEnvio['NRTPAMBIENTE']; //$tpAmb;
                        $dados['PROCEMI'] = $this->_procEmi;
                        $dados['VERPROC'] = $this->_verProc;
                    }

                    /* Formata os dados do evento conforme o objeto de descrição json*/
                    $dadosXml = $this->percorreAgrupadores($factory, $this->_projects, $dados, $this->_helpers);
                    
                    return $dadosXml;
                } else {
                    return false;
                }
            } else {
                return false;
            }
            return false;
        } catch(\Exception $e) {
            throw $e;
        }
    }

    /**
     * Percorre a estrutura json do evento e gera um array utilizando os valores retornados dos métodos do repositório
     * Para cada nível do json chama novamente a função com os campos do agrupador
     * Caso o agrupador seja um array, busca novos dados para ele e chama a função novamente com estes dados
     */
    private function percorreAgrupadores($factory, $agrupadores, $dadosAgrupador, $regras) {
        $dadosXml = null;
        foreach ($agrupadores as $nmAgrupador => $vrAgrupador) {
            if (is_array($vrAgrupador)) {
                if (isset($vrAgrupador[0])) {
                    // Para os itens de grupo que podem se repetir mais de uma vez
                    // Se for array de array busca os dados para preencher -> Ocorrencia maior que 1 no layout do eSocial
                    $dadosXml[$nmAgrupador] = array();

                    // Método dinâmico para buscar os dados de parte do evento (Ex: getDadosS1000softwareHouse)
                    $method = $this->_methodGetDados . $this->_nmEvento . $nmAgrupador;
                    if (method_exists($factory, $method)) {
                        $dadosAux = $factory->$method($dadosAgrupador);
                    } else {
                        $dadosAux = array();
                    }
                    // Percorre os dados retornandos setando na propriedade do evento
                    foreach ($dadosAux as $dados) {
                        array_push($dadosXml[$nmAgrupador],$this->percorreAgrupadores($factory, $vrAgrupador[0], $dados, $regras[strtoupper($nmAgrupador)]));
                    }
                    
                    
                    if (isset($regras[strtoupper($nmAgrupador)])) {
                        $this->executaRegras($nmAgrupador, $regras[strtoupper($nmAgrupador)], $dadosAgrupador, $dadosXml);
                    }
                    
                    
                } else {
                    // Caso o evento possua inclusão/alteração/exclusão verifica o tipo do envio a ser gerado
                    if (($nmAgrupador == 'inclusao' || $nmAgrupador == 'alteracao' || $nmAgrupador == 'exclusao') && ($nmAgrupador != $this->_idOperacao)) {
                        continue;
                    }
                    // Para os itens de grupo que não se repetem
                    $dadosXml[$nmAgrupador] = $this->percorreAgrupadores($factory, $vrAgrupador, $dadosAgrupador, $regras);
                    if ($this->agrupadorIsEmpty($dadosXml[$nmAgrupador])) {
                        unset($dadosXml[$nmAgrupador]);
                    }
                }
            } else {
                // Busca o valor no array de dados para os itens do tipo atributo e elemento
                $dadosXml[strtoupper($nmAgrupador)] = $this->getValorAgrupador($dadosAgrupador, $vrAgrupador);
                // Executas regras do campo
                if (isset($regras[strtoupper($vrAgrupador)])) {
                    $this->executaRegras(strtoupper($nmAgrupador), $regras[strtoupper($vrAgrupador)], $dadosAgrupador, $dadosXml);
                }
            }
        }
        return $dadosXml;
    }

    /**
     * Retorna o valor do agrupador conforme o array de dados
     */
    private function getValorAgrupador($dadosAgrupador, $vrAgrupador) {
        foreach ($dadosAgrupador as $name => $value) {
            if ($name == $vrAgrupador) {
                return $value;
            }
        }
        return null;
    }

    /**
     * Verifica se o agrupador está vazio
     */
    private function agrupadorIsEmpty($agrupador) {
        foreach ($agrupador as $index => $value) {
            if ((is_array($value) and !empty($value)) || (!is_array($value) && !is_null($value) && $value != '')) {
                return false;
            }
        }
        return true;
    }
    
    /**
     * Executa regras de validação
     */
    private function executaRegras($nmCampo, $regras, $dadosAgrupador, &$dadosXml) {
        // Quantidade de ocorrências do campo
        if (isset($regras['ocorrencia'])) {
            if ($regras['ocorrencia']['menor'] == 0 and (is_null($dadosXml[$nmCampo]) or (is_array($dadosXml[$nmCampo]) and empty($dadosXml[$nmCampo])))) {
                // Se a menor ocorrencia for 0 e o valor do campo vazio, retira o campo do array para não gerar tags desnecessárias no XML
                unset($dadosXml[$nmCampo]);
            }
            else if ($regras['ocorrencia']['menor'] == 1 and (is_null($dadosXml[$nmCampo]) || (is_array($dadosXml[$nmCampo]) && empty($dadosXml[$nmCampo])))) {
                // Se o campo for obrigatório verifica se existe condição para obrigatoriedade e lança mensagem de erro
                $dsCampo = isset($this->tagsTrad[$regras['descricao']]) ? $this->tagsTrad[$regras['descricao']] : $nmCampo;
                if (isset($regras['ocorrencia']['condicao'])) {
                    $valor = $dadosXml[$nmCampo];
                    if (eval("return " . $regras['ocorrencia']['condicao'] . ";")) {
                        MsgProcesso::getInstance()->addError($dsCampo . ' deve ser informado(a) {'.$nmCampo.'}');
                    }
                } else {
                    MsgProcesso::getInstance()->addError($dsCampo . ' deve ser informado(a) {'.$nmCampo.'}');
                }
            }
        }
        
        // Executa outras regras de validação
        if (isset($regras['regras'])) {
            // Valor do campo para ser utilizado na regra
            $valor = $dadosXml[$nmCampo];
            foreach ($regras['regras'] as $regra) {
                if (eval("return " . $regra['condicao'] . ";")) {
                    MsgProcesso::getInstance()->addError($regra['retorno']);
                    //throw new \Exception($regra['retorno']);
                }
            }
        }
    }

}