<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;

class EsoSimplificadoService {
    
    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        
        $this->eSocialRepository = new \HCM\Geral\Repository\ESocialRepository($this->entityManager->getConnection(), $this->entityManager, $this->hcmParameters);
    }

    public function beforeDeleteEvento($row) {
        try {
            if ($row['NMEVENTO'] == 'S-3000' || $row['IDOPERACAO'] == 'E' || $row['IDOPERACAO'] == 'R') {
                $enviosOrig = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO)->findBy(array('nrorg' => $row['NRORG'], 'nrenvioexcl' => $row['NRENVIO']));

                foreach ($enviosOrig as $envioOrig) {
                    $envioOrig->setNrenvioexcl();
                    $this->entityManager->persist($envioOrig);
                    $this->entityManager->flush();
                }
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function inativaEvento($nrorg, $nrenvio){
        $evento = $this->entityManager->getRepository(Repositories::ESO_NAOPERIODICO)->findBy(array('nrorg' => $row['NRORG'], 'nrenvioexcl' => $row['NRENVIO']));
        if ($evento) {
            $evento->setIdativo('N');
            $evento->setDtultatu(DateUtil::getDataAtual());
            $evento->setCdoperultatu($this->hcmParameters->getCdoperador());
            $evento->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $this->entityManager->persist($evento);
            $this->entityManager->flush();
        }
    }

    /**
     * Retorna o xml de envio, retorno de envio e retorno de consulta dos eventos
     * @param $nrorg
     * @param $xmlFile
     * @param $idEvento
     * @return array
     */
    public function getXmlFiles($nrorg, $xmlFile, $idEvento, $dsDirXml, $dsDirXmlFilesPath=null) {
        try {$file = $this->hcmParameters->getFileServerDirectory() . $dsDirXml;
            error_reporting(E_ALL ^ E_WARNING);
            if (fopen($file, 'r')) {
                $xml = file_get_contents($file);
            } else {
                throw new \Exception(Messages::FILE_NOT_FOUND);
            }
            error_reporting(E_ALL ^ E_NOTICE);
            return $xml;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setXmlFileToDataSet($xmlString) {
        try {
            $json = $this->parseXmlToJson($xmlString);
            $dataSet = $this->translateESocialJson($json);
            return $dataSet;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function parseXmlToJson($xmlFileContents) {
        try {
    		$xmlFileContents = str_replace(array("\n", "\r", "\t"), '', $xmlFileContents);
    		$xmlFileContents = trim(str_replace('"', "'", $xmlFileContents));
    		$simpleXml = simplexml_load_string($xmlFileContents);
    		$json = json_encode($simpleXml);
    		return $json;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialJson($json) {
        try {
            $rows = array();
    		$this->translateESocialTag(json_decode($json,true), null, $rows);
    		return $rows;
        } catch(\Exception $e) {
            throw $e;
        }
	}
	
	public function translateESocialTag($tag, $parent, &$rows) {
        try {
            $this->_tagsTrad = array();
            $this->_tagsOpc = array();
            $tagsTrad = $this->eSocialRepository->getTagName($this->hcmParameters->getNrorgPadrao() ? $this->hcmParameters->getNrorgPadrao() : 0);
            foreach ($tagsTrad as $tag) {
                $this->_tagsTrad[$tag['DSTAGNAME']] = $tag['DSTAGTRADUCAO'] != '' ?  $tag['DSTAGTRADUCAO'] : $tag['DSTAGNAME'];
                $cdOpcoes = $tag['DSCDOPCTAG'] ? explode(';', $tag['DSCDOPCTAG']) : null;
                $nmOpcoes = $tag['DSNMOPCTAG'] ? explode(';', $tag['DSNMOPCTAG']) : null;
                if($cdOpcoes){
                    foreach($cdOpcoes as $index => $cdOpcao){
                        $this->_tagsOpc[$tag['DSTAGNAME']][$cdOpcao] = $nmOpcoes[$index];
                    }
                }
            }
            
            $this->_tagNivel ++;
	        $id = count($rows) + 1;
    		foreach ($tag as $name => $value) {
    		    if ($name === 'Signature') {
    		        // Não traduz a assinatura do evento
    		        continue;
    		    }
    		    elseif (is_array($value)) {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (is_int($name) ? $name + 1 : (isset($this->_tagsTrad[$name]) ? $this->_tagsTrad[$name] : $name)),
    		            'VALUE' => null,
    		            '_opened' => true,
    		            'NIVEL' => $this->_tagNivel
		            );
    		        $this->translateESocialTag($value, $id, $rows);
    		        $id = count($rows) + 1;
    		    } else {
    		        $rows[] = array(
    		            'ID' => $id,
    		            'PARENT' => $parent,
    		            'NAME' => (isset($this->_tagsTrad[$name]) ? $this->_tagsTrad[$name] : $name),
    		            'VALUE' => array_key_exists($name, $this->_tagsOpc) ? $value.' - '.$this->_tagsOpc[$name][$value] : $value,
    		            '_opened' => true,
    		            'NIVEL' => $this->_tagNivel
		            );
		            $id = $id + 1;
    		    }
    		}
        } catch(\Exception $e) {
            throw $e;
        }
	}

}