<?php
namespace HCM\Geral\Services;

use HCM\Geral\Repository\PreFechamentoDctfWebRepository;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;

class PreFechamentoDctfWebService 
{
    public function __construct(PreFechamentoDctfWebRepository $preFechamentoDctfWebRepository, HCMParameters $hcmParameters)
    {
        $this->preFechamentoDctfWebRepository = $preFechamentoDctfWebRepository;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getContribuicaoPrevidenciariaSegurado($parameters)
    {
        $results = $this->preFechamentoDctfWebRepository->getContribuicaoPrevidenciariaSegurado($parameters);
        for($i=0; $i<count($results); $i++)
        {
            $results[$i]['CONCLUSAO'] = $this->getConclusao($results[$i]);
            $results[$i]['CNPJRAIZ'] = substr($results[$i]['CNPJLEGAL'], 0,8);
        }
        return $results;
    }
    
    public function getConclusao($params)
    {
        if($this->possuiDiferenca($params, 'BASE'))
        {
            $conclusao = 'Diferença na Base';
        }
        else if($this->possuiDiferenca($params, 'SEM_DESCONTO'))
        {
            $conclusao = 'Sem Desconto de INSS';
        }
        else if($this->possuiDiferenca($params, 'DESCONTO'))
        {
            $conclusao = 'Diferença no Desconto';
        }
        else if($this->possuiDiferenca($params, 'ARREDONDAMENTO'))
        {
            $conclusao = 'Arredondamento';
        } 
        else if($params['IDSTATUS'] == 'M')
        {
            $conclusao = 'Sem retorno do eSocial';
        } 
        else 
        {
            $conclusao = 'Sem Diferença';
        }
        return $conclusao;
    }
    
    public function possuiDiferenca($params, $flag)
    {
        $base               = $params['BASE'];
        $baseEsocial        = $params['VRBASE_ESO'];
        $desconto           = $params['INSS'];
        $descontoEsocial    = $params['VRCALCULADO_ESO'];
        $diferencaBase      = $params['DIFERENCA_BASE'];
        $diferencaDesconto  = $params['DIFERENCA_DESCONTO'];
        $status             = $params['IDSTATUS'];
   
        switch ($flag) 
        {
            case 'BASE':
                $result = ($base != $baseEsocial && $diferencaBase != '.01') && $status != 'M';
                break;
            case 'DESCONTO':
                $result = ($desconto != $descontoEsocial && $diferencaDesconto != '.01') && $status != 'M';
                break;
            case 'ARREDONDAMENTO':
                $result = ($base != $baseEsocial || $desconto != $descontoEsocial) &&
                          ($diferencaBase == '.01' || $diferencaDesconto == '.01') && $status != 'M';
                break;
            case 'SEM_DESCONTO':
                $result = ($desconto == null && $descontoEsocial == null || $desconto == 0 && $descontoEsocial == 0) || 
                          ($desconto == 0 && $descontoEsocial == null || $desconto == null && $descontoEsocial == 0) && $status != 'M';
                break;
            default:
                $result = false;
        }
        return $result;
    }
}