<?php
// Colocar arquivo no backend/temp
require_once __DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'backend' . DIRECTORY_SEPARATOR . 'bootstrap.php';

function main() 
{
    ini_set('memory_limit','2048M');

    $leiaute_path = __DIR__.DIRECTORY_SEPARATOR."..".DIRECTORY_SEPARATOR."..".DIRECTORY_SEPARATOR."backend".DIRECTORY_SEPARATOR."temp".DIRECTORY_SEPARATOR."esocial".DIRECTORY_SEPARATOR."leiautes";
    $versao = "2.4.01";
    $arquivo = "Leiautes do eSocial v2.4.01 Beta.xls";

    //$importador = new \Folha\Logic\ESocial\Importacao\ImportacaoLeiaute($leiaute_path, $versao, $arquivo);
    //$result = $importador->preparaInformacao();

    if (!is_file($leiaute_path.DIRECTORY_SEPARATOR.$arquivo)) {
        echo "Arquivo $arquivo não encontrado.\n";
        die;
    } else {
        echo "Lendo arquivo $arquivo...\n";
    }

    $handle = new ManipulacaoExcel($leiaute_path.DIRECTORY_SEPARATOR.$arquivo);
    $info = $handle->preparaDados();
    echo "Dados das planilhas lidos com sucesso!!\n";

    echo "Gravando dados...\n";
    $persist = new Persistencia($info);
    $persist->persisteTabelas($versao);

    $result = "Leiautes gerados com sucesso!!!\n";

    echo $result;
}

class ManipulacaoExcel
{
    private $file;

    private $eventos;
    private $resumos;
    private $registros;

    private $leiaute;

    private $lista_eventos;

    public function __construct($arquivo)
    {
        $this->file = \PHPExcel_IOFactory::load($arquivo);

        $this->eventos = array();
        $this->resumos = array();
        $this->registros = array();

        $this->leiaute = array();

        $this->lista_eventos = array();
    }

    public function preparaDados()
    {
        try {
            self::carregaEventos();
            self::carregaResumos();
            self::carregaRegistros();

            $this->leiaute = array(
                'EVENTOS' => $this->eventos,
                'RESUMOS' => $this->resumos,
                'REGISTROS' => $this->registros
            );

            return $this->leiaute;
        } catch (\Exception $e) {
            throw new \Exception("hrm_nenhum_dado_encontrado");
        }
    }

    private function carregaEventos()
    {
        try {
            $this->file->setActiveSheetIndexByName('Eventos');

            for ($row = 2; !is_null($this->file->getActiveSheet()->getCell('A'.$row)->getValue()); $row++) {
                $nmevento = $this->file->getActiveSheet()->getCell('A'.$row)->getValue();
                $grupo = $this->file->getActiveSheet()->getCell('B'.$row)->getValue();

                $this->eventos[] = array(
                    'DESCRICAO' => $nmevento,
                    'NOME' => substr($nmevento, 0, 6),
                    'IDGRUPEVENT' => $grupo
                );

                $this->lista_eventos[] = substr($nmevento, 0, 6);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function carregaResumos()
    {
        try {
            $this->file->setActiveSheetIndexByName('Resumos');
            $activeSheet = $this->file->getActiveSheet();

            $listaEventos = $this->lista_eventos;

            for ($row = 1; !empty($listaEventos); $row++) {
                $value = $activeSheet->getCell('A'.$row)->getValue();

                /*Extrai o nome do evento do título da tabela (Ex: S-1000 - Informações do Empregador/Contribuinte)*/
                $eventoIndex = array_search(substr($value,0,6), $listaEventos);
                if ($eventoIndex !== false) {
                    $nmevento = $listaEventos[$eventoIndex];

                    $endRow = self::percorreResumoEvento($activeSheet, $nmevento, $row);
                    $row = $endRow;

                    unset($listaEventos[$eventoIndex]);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function percorreResumoEvento($activeSheet, $nmevento, $eventoRow)
    {
        /*Pula as linhas com o nome do evento, o título da tabela e os nomes das colunas*/
        $startRow = $eventoRow + 3;

        for ($row=$startRow; !is_null($activeSheet->getCell('A'.$row)->getValue()); $row++) {

            $nmReg = $activeSheet->getCell('A'.$row)->getValue();
            $this->resumos[$nmevento][] = array(
                'REGISTRO' => $activeSheet->getCell('A'.$row)->getValue(),
                'PAI' => $activeSheet->getCell('B'.$row)->getValue(),
                'NIVEL' => $activeSheet->getCell('C'.$row)->getValue(),
                'DESCRICAO' => str_replace('\n',' ',$activeSheet->getCell('D'.$row)->getValue()),
                'OCORRENCIA' => $activeSheet->getCell('E'.$row)->getValue(),
                'CHAVE' => str_replace('\n',' ',$activeSheet->getCell('F'.$row)->getValue()),
                'CONDICAO' => str_replace('\n',' ',$activeSheet->getCell('G'.$row)->getValue())
            );

            $lastRow = $row;
        }

        return $lastRow;
    }

    private function carregaRegistros()
    {
        try {
            $this->file->setActiveSheetIndexByName('Registros');
            $activeSheet = $this->file->getActiveSheet();

            $listaEventos = $this->lista_eventos;

            for ($row = 1; !empty($listaEventos); $row++) {
                $value = $activeSheet->getCell('A'.$row)->getValue();

                /*Extrai o nome do evento do título da tabela (Ex: Registros do evento S-4999 - Adesão antecipada ao eSocial)*/
                $eventoIndex = array_search(substr($value,20,6), $listaEventos);
                if ($eventoIndex !== false) {
                    $nmevento = $listaEventos[$eventoIndex];

                    $endRow = self::percorreRegistroEvento($activeSheet, $nmevento, $row);
                    $row = $endRow;

                    unset($listaEventos[$eventoIndex]);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function percorreRegistroEvento($activeSheet, $nmevento, $eventoRow)
    {
        /*Pula as linhas com o nome do evento e os nomes das colunas*/
        $startRow = $eventoRow + 2;

        for ($row=$startRow; !is_null($activeSheet->getCell('A'.$row)->getValue()); $row++) {

            $nrCampo = $activeSheet->getCell('A'.$row)->getValue();
            $this->registros[$nmevento][] = array(
                'NRCAMPO' => $activeSheet->getCell('A'.$row)->getValue(),
                'CAMPO' => $activeSheet->getCell('B'.$row)->getValue(),
                'PAI' => $activeSheet->getCell('C'.$row)->getValue(),
                'ELEMENTO' => $activeSheet->getCell('D'.$row)->getValue(),
                'TIPO' => $activeSheet->getCell('E'.$row)->getValue(),
                'OCORRENCIA' => $activeSheet->getCell('F'.$row)->getValue(),
                'TAMANHO' => $activeSheet->getCell('G'.$row)->getValue(),
                'DECIMAL' => $activeSheet->getCell('H'.$row)->getValue(),
                'DESCRICAO' => $activeSheet->getCell('I'.$row)->getValue()
            );

            $lastRow = $row;
        }

        return $lastRow;
    }
}

class Persistencia
{
    private $esoEvento = 'Teknisa\Model\Gestaopessoas\Entities\EsoEvento';
    private $esoResumo = 'Teknisa\Model\Gestaopessoas\Entities\EsoResumo';
    private $esoRegistro = 'Teknisa\Model\Gestaopessoas\Entities\EsoRegistro';
    
    private $eventos;
    private $resumos;
    private $registros;

    private $nrorgpadrao;

    /**
     * @param $caminho          Pasta na qual se encontram os arquivos de leiaute
     * @param $versao           Versão do E-Social
     * @param $arq_eventos      Nome do arquivo com a descrição dos eventos
     * @param $arq_resumos      Nome do arquivo com os resumos dos eventos
     * @param $arq_registros    Nome dos arquivos com os registros do eventos
     * @throws \Exception
     */
    public function __construct($leiautes)
    {
        try {
            $this->validaNrEventos($leiautes);

            $this->eventos = $leiautes['EVENTOS'];
            $this->resumos = $leiautes['RESUMOS'];
            $this->registros = $leiautes['REGISTROS'];

            $this->nrorgpadrao = 0;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function persisteTabelas($versao)
    {
        try {
            \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();

            self::insereGpeEsocial($versao);
            //self::geraArvoreRegistro($versao);

            \Setforms2\Shared\Registry::getEntityManager()->commit();
        } catch (\Exception $e) {
            \Setforms2\Shared\Registry::getEntityManager()->rollback();
            throw $e;
        }
    }

    /**
     * Validada se a quantidade de eventos em cada leiaute está consistente.
     *
     * @param $leiautes
     *
     * @return bool
     * @throws \Exception
     */
    private function validaNrEventos($leiautes)
    {
        try {
            $nrEventos = count($leiautes['EVENTOS']);
            $nrResumos = count($leiautes['RESUMOS']);
            $nrRegistros = count($leiautes['REGISTROS']);

            if (($nrEventos != $nrResumos) or ($nrResumos != $nrRegistros) or ($nrEventos != $nrRegistros)) {
                throw new \Exception('Quantidade de eventos errada');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insereGpeEsocial($versao)
    {
        try {
            $maxEvento = current(\Setforms2\Shared\Registry::getEntityManager()->getRepository($this->esoEvento)->findBy(array('nrorg' => $this->nrorgpadrao), array('nrevento' => 'DESC')));
            $nrEvento = $maxEvento->getNrevento() + 1;
            
            $maxResumo = current(\Setforms2\Shared\Registry::getEntityManager()->getRepository($this->esoResumo)->findBy(array('nrorg' => $this->nrorgpadrao), array('nrresumo' => 'DESC')));
            $nrResumo = $maxResumo->getNrresumo() + 1;
            
            $maxRegistro = current(\Setforms2\Shared\Registry::getEntityManager()->getRepository($this->esoRegistro)->findBy(array('nrorg' => $this->nrorgpadrao), array('nrregistro' => 'DESC')));
            $nrRegistro = $maxRegistro->getNrregistro() + 1;
                
            foreach ($this->eventos as $evento) {

                $esoEventoOld = \Setforms2\Shared\Registry::getEntityManager()->getRepository($this->esoEvento)->findOneBy(array('nmevento' => $evento['NOME'], 'dsversao' => $versao));
                if (is_object($esoEventoOld)) {
                    throw new \Exception('Existe versão para o evento ' . $evento['NOME']);
                }
                
                echo "Gravando evento " . $evento['NOME'] . "...";
                
                $gpeEsocial = new $this->esoEvento();
                $gpeEsocial->setNrevento($nrEvento);
                $gpeEsocial->setDsevento($evento['DESCRICAO']);
                $gpeEsocial->setNmevento($evento['NOME']);
                $gpeEsocial->setIdgrupo($evento['IDGRUPEVENT']);
                $gpeEsocial->setDsversao($versao);
                $gpeEsocial->setNrorg($this->nrorgpadrao);
                $gpeEsocial->setDtinclusao(\HCM\Util\DateUtil::getDataAtual(false));
                $gpeEsocial->setCdoperinclusao('1');
                $gpeEsocial->setNrorginclusao($this->nrorgpadrao);
                $gpeEsocial->setDtultatu(\HCM\Util\DateUtil::getDataAtual(false));
                $gpeEsocial->setCdoperultatu('1');
                $gpeEsocial->setNrorgultatu($this->nrorgpadrao);
                $gpeEsocial->setIdativo('S');
                \Setforms2\Shared\Registry::getEntityManager()->persist($gpeEsocial);
                \Setforms2\Shared\Registry::getEntityManager()->flush();

                $nrResumo = self::insereGpeEsocialResumo($nrEvento, $this->resumos[$evento['NOME']], $nrResumo);
                $nrRegistro = self::insereGpeEsocialRegistro($nrEvento, $this->registros[$evento['NOME']], $nrRegistro);
                
                echo " OK!\n";
                $nrEvento++;
            }
            
            \Setforms2\Shared\Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insereGpeEsocialResumo($nrEvento, $resumos, $nrResumo)
    {
        try {
            
            echo " Resumos...";
            
            foreach ($resumos as $resumo) {
                $gpeEsocialResumo = new $this->esoResumo();
                $gpeEsocialResumo->setNrresumo($nrResumo);
                $gpeEsocialResumo->setNrevento($nrEvento);
                $gpeEsocialResumo->setNmregistro(preg_replace('~[\n]+~','',$resumo['REGISTRO']));
                $gpeEsocialResumo->setNmregisropai(preg_replace('~[\n]+~','',$resumo['PAI']));
                $gpeEsocialResumo->setNrnivel($resumo['NIVEL']);
                $gpeEsocialResumo->setDsresumo(preg_replace('~[\n]+~',' ',$resumo['DESCRICAO']));
                $gpeEsocialResumo->setDsocorrencia($resumo['OCORRENCIA']);
                $gpeEsocialResumo->setDschave(preg_replace('~[\n]+~',' ',$resumo['CHAVE']));
                $gpeEsocialResumo->setDscondicao(preg_replace('~[\n]+~',' ',$resumo['CONDICAO']));
                $gpeEsocialResumo->setNrorg($this->nrorgpadrao);
                $gpeEsocialResumo->setDtinclusao(\HCM\Util\DateUtil::getDataAtual(false));
                $gpeEsocialResumo->setCdoperinclusao('1');
                $gpeEsocialResumo->setNrorginclusao($this->nrorgpadrao);
                $gpeEsocialResumo->setDtultatu(\HCM\Util\DateUtil::getDataAtual(false));
                $gpeEsocialResumo->setCdoperultatu('1');
                $gpeEsocialResumo->setNrorgultatu($this->nrorgpadrao);
                $gpeEsocialResumo->setIdativo('S');
                \Setforms2\Shared\Registry::getEntityManager()->persist($gpeEsocialResumo);
                //\Setforms2\Shared\Registry::getEntityManager()->flush();
                $nrResumo++;
            }
            
            return $nrResumo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function insereGpeEsocialRegistro($nrEvento, $registros, $nrRegistro)
    {
        try {
            
            echo " Registros...";
            
            foreach ($registros as $registro) {
                $gpeEsocialRegistro = new $this->esoRegistro();
                $gpeEsocialRegistro->setNrregistro($nrRegistro);
                $gpeEsocialRegistro->setNrevento($nrEvento);
                $gpeEsocialRegistro->setNrcampo($registro['NRCAMPO']);
                $gpeEsocialRegistro->setNmcampo(preg_replace('~[\n]+~','',$registro['CAMPO']));
                $gpeEsocialRegistro->setNmcampopai(preg_replace('~[\n]+~','',$registro['PAI']));
                $gpeEsocialRegistro->setDselemento($registro['ELEMENTO']==="CG" ? "G" : $registro['ELEMENTO']);
                $gpeEsocialRegistro->setDstipo($registro['TIPO']);
                $gpeEsocialRegistro->setDsocorrencia($registro['OCORRENCIA']);
                $gpeEsocialRegistro->setNrtamanho($registro['TAMANHO']==="-" ? null : $registro['TAMANHO']);
                $gpeEsocialRegistro->setNrdecimais($registro['DECIMAL']==="-" ? null : $registro['DECIMAL']);
                $gpeEsocialRegistro->setDsregistro(preg_replace('~[\n]+~',' ',$registro['DESCRICAO']));
                $gpeEsocialRegistro->setNrorg($this->nrorgpadrao);
                $gpeEsocialRegistro->setDtinclusao(\HCM\Util\DateUtil::getDataAtual(false));
                $gpeEsocialRegistro->setCdoperinclusao('1');
                $gpeEsocialRegistro->setNrorginclusao($this->nrorgpadrao);
                $gpeEsocialRegistro->setDtultatu(\HCM\Util\DateUtil::getDataAtual(false));
                $gpeEsocialRegistro->setCdoperultatu('1');
                $gpeEsocialRegistro->setNrorgultatu($this->nrorgpadrao);
                $gpeEsocialRegistro->setIdativo('S');
                \Setforms2\Shared\Registry::getEntityManager()->persist($gpeEsocialRegistro);
                //\Setforms2\Shared\Registry::getEntityManager()->flush();
                $nrRegistro++;
            }
            
            return $nrRegistro;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    private function geraArvoreRegistro($versao)
    {
        $esoEvento = \Setforms2\Shared\Registry::getEntityManager()->getRepository($this->esoEvento)->findOneBy(
            array('dsversao' => $versao), array('nrevento' => 'ASC')
        );

        foreach ($esoEvento as $evento) {
            $nrPai = null;
            $gpeEsocialRegistro = \Setforms2\Shared\Registry::getEntityManager()->getRepository($this->esoRegistro)->findBy(
                array('nrevento' => $evento->getNrevento()), array('nrcampo' => 'ASC')
            );

            foreach ($gpeEsocialRegistro as $registro) {
                if (is_null($registro->getNmcampopai())) {
                    $nrPai = $registro->getNrcampo();
                } else {
                    $nrRegistroPai = \Setforms2\Shared\Registry::getEntityManager()->getRepository($this->esoRegistro)->findOneBy(
                        array('nrevento' => $evento->getNrevento(), 'nmcampo' => $registro->getNmcampopai(), )
                    );
                    $registro->setNrregistropai($nrPai);
                }
            }
        }
    }
}

main();